/*
 * Copyright (C) 2024 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#pragma once

#include "MatchResult.h"
#include "SlotVisitorMacros.h"
#include "WriteBarrier.h"

namespace JSC {

class JSString;
class JSRopeString;
class RegExp;

// This class is used to track the cached results of the last match
// on a substring with global and atomic pattern regexp. It's
// looking for the code pattern:
//
//     str.substring(0, 50).match(regExp);
//     str.substring(0, 60).match(regExp);
class RegExpSubstringGlobalAtomCache {
public:
    JSValue collectMatches(JSGlobalObject*, JSRopeString* substring, RegExp*);

    DECLARE_VISIT_AGGREGATE;

private:
    WriteBarrier<JSString> m_lastSubstringBase;
    unsigned m_lastSubstringOffset;
    unsigned m_lastSubstringLength;

    WriteBarrier<RegExp> m_lastRegExp;
    size_t m_lastNumberOfMatches { 0 };
    size_t m_lastMatchEnd { 0 };
    MatchResult m_lastResult { };
};

} // namespace JSC
