/*
 * Copyright (C) 2016 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "IDBGetAllResult.h"

#include <wtf/CrossThreadCopier.h>
#include <wtf/HashSet.h>
#include <wtf/TZoneMallocInlines.h>

namespace WebCore {

WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(IDBGetAllResult);

IDBGetAllResult::IDBGetAllResult(const IDBGetAllResult& that, IsolatedCopyTag)
{
    isolatedCopy(that, *this);
}

IDBGetAllResult IDBGetAllResult::isolatedCopy() const
{
    return { *this, IsolatedCopy };
}

void IDBGetAllResult::isolatedCopy(const IDBGetAllResult& source, IDBGetAllResult& destination)
{
    destination.m_type = source.m_type;
    destination.m_keys = crossThreadCopy(source.m_keys);
    destination.m_values = crossThreadCopy(source.m_values);
    destination.m_keyPath = crossThreadCopy(source.m_keyPath);
}

void IDBGetAllResult::addKey(IDBKeyData&& key)
{
    m_keys.append(WTFMove(key));
}

void IDBGetAllResult::addValue(IDBValue&& value)
{
    m_values.append(WTFMove(value));
}

const Vector<IDBKeyData>& IDBGetAllResult::keys() const
{
    return m_keys;
}

const Vector<IDBValue>& IDBGetAllResult::values() const
{
    return m_values;
}

Vector<String> IDBGetAllResult::allBlobFilePaths() const
{
    ASSERT(m_type == IndexedDB::GetAllType::Values);

    HashSet<String> pathSet;
    for (auto& value : m_values) {
        for (auto& path : value.blobFilePaths())
            pathSet.add(path);
    }

    return copyToVector(pathSet);
}

} // namespace WebCore
